from django.db import models
from wagtail.documents.models import Document as WagtailDocument
from wagtail.search import index

from wagtail_textract.handlers import async_transcribe_document


class TranscriptionMixin(models.Model):
    """Mixin class with transcription field and save method."""
    transcription = models.TextField(default='', blank=True)

    class Meta:
        """Don't create a table, this model is only for subclassing."""
        abstract = True

    def save(self, **kwargs):
        """Asynchronously transcribe the file."""
        transcribe = kwargs.pop('transcribe', True)
        super(TranscriptionMixin, self).save(**kwargs)
        if transcribe:
            async_transcribe_document(self)


class Document(TranscriptionMixin, WagtailDocument):
    """Include transcription in search_fields."""
    search_fields = WagtailDocument.search_fields + [
        index.SearchField(
            'transcription',
            partial_match=False,
        ),
    ]



from django.apps import AppConfig


class WagtailTextractAppConfig(AppConfig):
    """Wagtail-Textract AppConfig."""
    name = 'wagtail_textract'
    label = 'wagtail_textract'
    verbose_name = "Wagtail-Textract Search integration"



from django.core.management.base import BaseCommand

from wagtail.documents.models import get_document_model


class Command(BaseCommand):
    """Extract text from all Documents."""

    def handle(self, *args, **options):
        """Extract text from all Documents."""
        for document in get_document_model().objects.all():
            self.stdout.write("Transcribing %s" % document)
            document.transcribe()



# Generated by Django 2.0.4 on 2018-05-03 11:17

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('wagtail_textract', '0002_auto_20180502_1303'),
    ]

    operations = [
        migrations.AlterField(
            model_name='document',
            name='transcription',
            field=models.TextField(blank=True, default=''),
        ),
    ]



from wagtail.documents.models import get_document_model


def test_document_class():
    """Test that the Document model has the required (search) field.

    Actually, this only tests if Wagtails WAGTAILDOCS_DOCUMENT_MODEL
    still works, and that our test Django settings are correct.
    """
    Document = get_document_model()
    assert hasattr(Document, 'transcription')
    assert 'transcription' in [f.field_name for f in Document.search_fields]



import asyncio
import logging
import textract

logger = logging.getLogger(__name__)
loop = asyncio.get_event_loop()


def transcribe_document(document):
    """Store the Document file's text in the transcription field."""
    try:
        text = textract.process(document.file.path).strip()
        if not text:
            logger.debug('No text found, falling back to tesseract.')
            text = textract.process(
                document.file.path,
                method='tesseract',
            ).strip()

    except Exception as err:
        text = None
        logger.error(
            'Text extraction error with file {file}: {message}'.format(
                file=document.filename,
                message=str(err),
            )
        )

    if text:
        document.transcription = text.decode()
        document.save(transcribe=False)
        print("Saved transcription: %s" % text)
    else:
        logger.error('No text found.')


def async_transcribe_document(document):
    """Defer transcription to an asyncio executor."""
    loop.run_in_executor(None, transcribe_document, document)



# Generated by Django 2.0.4 on 2018-05-08 09:42

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('wagtail_textract', '0003_auto_20180503_1117'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='document',
            options={},
        ),
    ]



#!/usr/bin/env python
try:
    from setuptools import setup, find_packages
except ImportError:
    from distutils.core import setup


# Hack to prevent "TypeError: 'NoneType' object is not callable" error
# in multiprocessing/util.py _exit_function when setup.py exits
# (see http://www.eby-sarna.com/pipermail/peak/2010-May/003357.html)
try:
    import multiprocessing  # NOQA
except ImportError:
    pass


install_requires = [
    "wagtail>=2,<2.1",
    "textract",
]

tests_require = [
    'pytest',
    'pytest-django',
]

setup(
    name='wagtail-textract',
    version='0.1a2',
    description='Allow searching for text in Documents in the Wagtail content management system',
    author='Kees Hink',
    author_email='kees@fourdigits.nl',
    url='https://github.com/fourdigits/wagtail-textract',
    package_dir={'': 'src'},
    packages=find_packages('src'),
    include_package_data=True,
    license='BSD',
    long_description=open('README.md', 'r').read(),
    long_description_content_type='text/markdown',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Framework :: Django',
        'Framework :: Django :: 2.0',
        'Framework :: Wagtail',
        'Framework :: Wagtail :: 2',
        'Topic :: Internet :: WWW/HTTP :: Site Management',
    ],
    install_requires=install_requires,
    extras_require={
        'test': tests_require,
    },
    entry_points="""""",
    zip_safe=False,
    cmdclass={
    },
)



# Generated by Django 2.0.4 on 2018-05-08 09:45

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    replaces = [('wagtail_textract', '0001_initial'), ('wagtail_textract', '0002_auto_20180502_1303'), ('wagtail_textract', '0003_auto_20180503_1117'), ('wagtail_textract', '0004_auto_20180508_0942')]

    initial = True

    dependencies = [
        ('wagtaildocs', '0007_merge'),
    ]

    operations = [
        migrations.CreateModel(
            name='Document',
            fields=[
                ('document_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='wagtaildocs.Document')),
                ('transcription', models.TextField(blank=True, default='')),
            ],
            options={
                'verbose_name': 'document',
                'abstract': False,
            },
            bases=('wagtaildocs.document',),
        ),
    ]
